#$Author: martin $
#$Header: /home/martin/src/Perl/File/Edit/Edit.pm,v 1.8 2000/04/20 18:27:24 martin Exp martin $
#$Id: Edit.pm,v 1.8 2000/04/20 18:27:24 martin Exp martin $

package File::Edit;

use Carp;
use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);

@EXPORT = qw();
@EXPORT_OK = qw();

$VERSION = '0.3.1';


# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__


=head1 NAME

File::Edit - Edit a file by the line.

=head1 SYNOPSIS

 use File::Edit;
 
 $file  = Edit->new("Filename");

 $count = $file->grep("RegExp");

 $line  = $file->getline("RegExp" | Line Number);

 $num   = $file->delete("RegExp");
 
 $line  = $file->deleteline("RegExp" | Line Number);
 
 $file->insert("Insert String", Line Number);



=head1 DESCRIPTION

This Perl-Module helps the programmer to edit a file line by line. The
string to edit can be choosen by a regular expression.

The following methods are implemented:

 $file = Edit->new("Filename");

Create a new File::Edit-Object and bind it to file "Filename".
 
 
 $line = $file->grep("RegExp");

The method get reads the file an returns the number of lines where 
the regexp matches. 
 
 $line = $file->getline("RegExp" | LineNr);
 
If called with only a number, the method getline returns the line 
given by this number. If called with a regexp, it returns the FIRST 
line where the regexp matches.


 $line = $file->delete("RegExp");

The method delete delete the phrases where the regexp matches. It will
return the number of lines where it has delete the regexp.

 
 $match = $file->deleteline("RegExp"| LineNr);

Delete all lines are matching the regexp. The return-value is the number
of lines are deleted. If only called with a Number it will delete the line
LineNr and returns it.

 
 $file->insert("Insert String", Line Number);

This will insert the String "Insert String" at the line "Line Number". 
If given a "Line Number" of 0, the method will append the string to the 
file.


=head1 AUTHOR

Martin Werthmoeller, software@werthmoeller.de

=head1 SEE ALSO

perl(1).



=head1 COPYRIGHT 

Copyright (c) 2000 Martin Werthmoeller. All right reserved. This
program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut


sub new {
    my $class = shift;
    my $self  = { fname => shift };
    bless $self, $class;
}


sub _edit_open {
    my ($self) = shift;

    my $newfile = $self->{'fname'} . ".BAK";

    if (! -f $self->{'fname'}) { 
        open SOURCE, ">$self->{'fname'}";
    }

    open DRAIN, ">$newfile" or
         croak "$!: ", $newfile;
    open SOURCE, "<$self->{'fname'}" or
         croak "$!: ", $self->{'fname'};
}


sub _edit_close {
    my ($self) = shift;
    close DRAIN;
    close SOURCE;

    unlink $self->{'fname'} 
           or croak "$!: ", $self->{'fname'}, ".\n";
    my $newfile = $self->{'fname'} . ".BAK";
    rename $newfile, $self->{'fname'}
           or croak "$!: ", $newfile, ".\n";;
}

sub _read_open {
    my ($self) = shift;
    open SOURCE, "$self->{'fname'}";
}

sub _read_close {
    close SOURCE;
}

# grep returns the number of lines where regexp matches
sub grep {
    my ($self, $regexp) = @_;
    my $count = 0;

    $self->_read_open();
    foreach (<SOURCE>) {
        if ( $_ =~ /($regexp)/ ) {
            $count++;
        }
    }
    $self->_read_close();
    return $count;
}


# getline returns the first line where regexp matches
# or the line Line Number
sub getline {
    my ($self, $regexp) = @_;
 
    $self->_read_open();
    if ($regexp =~ /\d+/ ) {                    # return line Line Number
        my $linenr = 0;
        foreach (<SOURCE>) {
            $linenr++;
            if ($regexp == $linenr) {
                $self->_read_close;
                return $_;
            }
        }
    } else {                                    # return hash-ref
        foreach (<SOURCE>) {
            if ( $_ =~ /$regexp/ ) {
                $self->_read_close();
                return $_;
            }
        }
    }
}


# delete the phrase matches on regexp, return count of del
sub delete {
    my ($self, $regexp) = @_;
    my $count = 0;

    $self->_edit_open();
    foreach (<SOURCE>) {
        if ( $_ =~ s/$regexp//g ) {
            $count++;
        }
        print DRAIN $_;
    }
    $self->_edit_close();
    return $count;
}


# delete the FIRST line where regexp matches and returns it.
# delete the line LineNr and returns it.
sub deleteline {
    my ($self, $regexp) = @_;
    my $retval;
    $self->_edit_open();
    if ($regexp =~ /^\d+$/) {     # Delete line by number
        my $linenr;
        foreach (<SOURCE>) {
            $linenr++;
            if ($regexp == $linenr) {
                $retval = $_;
                next;
            }
            print DRAIN $_;
        }
        $self->_edit_close();
        return $retval;
    } else {                    # Delete line by regexp
        foreach (<SOURCE>) {
            if ($_ =~ /$regexp/g) {
                $retval = $_; 
                next;
            }
            print DRAIN $_;
        }
        $self->_edit_close();
        return $retval;
    }
}

# insert line $iline
sub insert {
    my ($self, $iline, $lnumber) = @_;
    my $linenr = 0;

    $self->_edit_open();
    foreach (<SOURCE>) {
        $linenr++;
        if ($linenr == $lnumber) {
            print DRAIN $iline, "\n";
        }
        print DRAIN $_;
    }
    if ($lnumber == 0) {
        print DRAIN $iline, "\n";
    }
    $self->_edit_close();
}
